use std::io;

use dump::{self, Kind};
use web::MyResponse;

use database::{Database, KeyDatabase, Query};

#[get("/debug?<q>")]
pub fn debug_info(
    db: rocket::State<KeyDatabase>,
    q: String,
) -> MyResponse {
    let query = match q.parse::<Query>() {
        Ok(query) => query,
        Err(_) => return MyResponse::bad_request_plain("bad request"),
    };
    let fp = match db.lookup_primary_fingerprint(&query) {
        Some(fp) => fp,
        None => return MyResponse::not_found_plain(query.describe_error()),
    };

    let armored_key = match db.by_fpr(&fp) {
        Some(armored_key) => armored_key,
        None => return MyResponse::not_found_plain(query.describe_error()),
    };

    let mut result = Vec::new();
    let dump_result = dump::dump(
        &mut io::Cursor::new(armored_key.as_bytes()),
        &mut result,
        false,
        false,
        None,
        32 * 4 + 80,
    );
    match dump_result {
        Ok(Kind::TPK) => {
            match String::from_utf8(result) {
                Ok(dump_text) => MyResponse::plain(dump_text),
                Err(e) => MyResponse::ise(e.into()),
            }
        },
        Ok(_) => MyResponse::ise(failure::err_msg("Internal parsing error!")),
        Err(e) => MyResponse::ise(e),
    }
}
